// #autoload
// #name = QuickCD
// #version = 1.1
// #date = December 30, 2002
// #category = Automation
// #author = Lou Cypher
// #warrior = LouCypher
// #email = asta_llama_lincoln@hotmail.com
// #web = http://deadzone.cjb.net
// #description = Quick CD Player Interface
// #status = beta
// #readme = scripts/autoexec/quickcd.txt
// #credit = Harold "LabRat" Brown - code used from his CD Player for Torque

function RedBookCallback(%type)
{
	error("Redbook: " @ %type);
	if(%type $= "PlayFinished")
      CDPlayer.playFinished();
}

if (!isobject(CDPlayer))
{
	new ScriptObject(CDPlayer) {
	   class = CDAudio;
	   currentTrack = 0;
	   dcurrentTrack = 1;
	   totalTracks = 0;
	   volume = $pref::Audio::musicVolume;
	   isPlaying = false;
	   isPaused = false;
	};
}

redbookOpen();
redbookSetVolume($pref::Audio::musicVolume);
CDPlayer.totalTracks = redbookGetTrackCount();

//----
function CDAudio::setVolume(%this)
{
	%this.volume = cdvolume.value;
	$pref::Audio::musicVolume = %this.volume;
	redbookSetVolume(%this.volume);
}

function CDAudio::playFinished(%this)
{
	%this.isPlaying = false;
	if ($pref::QuickCD::Shuffle) {
		%track = mFloor(getRandom(0,%this.getTrackCount()-1));
		%this.schedule(1500,"playTrack",%track);
	} else {
		if ($pref::QuickCD::Repeat){
			%this.currentTrack++;
      	   	if (%this.currentTrack == %this.getTrackCount()) {
				error("Last track!");
				%this.currentTrack = 0;
			}
			error("Track: " @ %this.currentTrack);
			%this.schedule(1500,"playTrack",%this.currentTrack);			
		} else {
      	   	if (%this.currentTrack == %this.getTrackCount()) %this.stop();
      	   	%this.currentTrack++;
			%this.schedule(1500,"playTrack",%this.currentTrack);
		}
	}
}

function CDAudio::play(%this)
{
   if(%this.isPlaying == true)
      return;

   %numTracks = %this.getTrackCount();
   %this.totalTracks = %numTracks;

   if(%numTracks == 0)
   {
      error(redbookGetLastError());
      return;
   }
   
   if (%this.currentTrack > %numTracks)
      %this.currentTrack = 0;
      
   %this.playTrack(%this.currentTrack);
}

function CDAudio::playTrack(%this, %track)
{
   if(%this.isPlaying == true)
      return;
	
   if(redbookPlay(%track) == false)
   {
      error(redbookGetLastError());
      return;  
   }
   
   %this.currentTrack = %track;
   %this.dcurrentTrack = %track+1;
   %this.isPlaying = true;
   %this.time(0);
}

function CDAudio::time(%this,%time) {
	%minutes=mfloor(%time/60);
	%seconds=%time-(%minutes*60);
	if (%seconds < 10) {
		%seconds = "0" @ %seconds;
	}
	cdplayer.timeleft=%minutes @ ":" @ %seconds;
	cdplayer.timeleftsec=%time;
	%time++;
	if(%this.isPlaying == true)
		%this.schedule(1000,"time",%time);
}

function CDAudio::stop(%this)
{
	redbookStop();
	%this.isPlaying = false;
}

function CDAudio::playNext(%this)
{
	if ($pref::QuickCD::Shuffle) %this.currentTrack = mFloor(getRandom(0,%this.getTrackCount()-1));
	else {
		%this.currentTrack++;
		if (%this.currentTrack > %this.totalTracks)
			%this.currentTrack = 0;
	}
	%this.dCurrentTrack = %this.currentTrack + 1;
	cdplayer.timeleft = "0:00";
	if (%this.isPlaying == true) {
		%this.isPlaying = false;
		%this.schedule(1250,"playTrack",%this.currentTrack);
	}
}

function CDAudio::playPrevious(%this)
{
	if ($pref::QuickCD::Shuffle) %this.currentTrack = mFloor(getRandom(0,%this.getTrackCount()-1));
	else {
		%this.currentTrack--;
		if (%this.currentTrack < 0)
			%this.currentTrack = (%this.totalTracks-1);
	}
	%this.dCurrentTrack = %this.currentTrack + 1;
	cdplayer.timeleft = "0:00";
	if (%this.isPlaying == true)
	{
		%this.isPlaying = false;
		%this.schedule(1250,"playTrack",%this.currentTrack);
	}
}

function CDAudio::getTrackCount(%this)
{
   return(redbookGetTrackCount());
}

//--------------------------------------------------------------------------
// - toggle 'visible' so this, and other, controls can determine if it is up
function activateQuickCD(%make)
{
	if(%make && !TaskHudDlg.isVisible())
		showQuickCD();
}

// Function that opens menu
function showQuickCD() 
{
    if (isObject(HM) && isObject(HudMover))
         QuickCDGUI.position = $hudposQuickCD;
	Canvas.pushDialog(QuickCD);
	QuickCDGUI.setVisible(true);
}

package QuickCD {

	function QuickCD::onWake ( %this ) {
		cdvolume.setValue( $pref::Audio::musicVolume );
	}

	function PlayGui::onWake( %this )
	{
		parent::onWake( %this );
    	if (isObject(HM) && isObject(HudMover))
    		hudmover::addhud(CDPlayerGui, "Quick CD");
	}

	function OptionsDlg::onWake( %this ) {
		if($QuickBind == false) {
			$RemapName[$RemapCount]="Quick CD";
			$RemapCmd[$RemapCount]="activateQuickCD";
			$RemapCount++;
			$QuickBind = true;
		}
		parent::onWake( %this );
	}

	function DispatchLaunchMode() {
		parent::DispatchLaunchMode();
		exec("gui/QuickCD.gui");
		cdvolume.setValue( $pref::Audio::musicVolume );
	}

    function LaunchToolbarDlg::onWake(%this)
    {
         parent::onWake(%this);
         LaunchToolbarMenu.add( 25, "QUICK CD" );
    }

	function LaunchToolbarMenu::onSelect(%this, %id, %text)
	{
		parent::onSelect(%this, %id, %text);
		switch(%id)
		{
			case 25:
				Canvas.pushDialog(QuickCD);
				playGui.add($IPCGUi);
		}
	}
}; // Common package
activatePackage(QuickCD);